@@ -64,6 +64,7 @@ static struct {
 	char *dhcp4_clientid;
 	char *dhcp4_hostname;
 	char *dhcp4_fqdn;
+	char *mud_url;
 	char *iid_str;
 	NMSettingIP6ConfigAddrGenMode addr_gen_mode;
 	char *logging_backend;
@@ -448,7 +449,7 @@ main (int argc, char *argv[])
 		fprintf (stderr,
 		         _("Ignoring unrecognized log domain(s) '%s' passed on command line.\n"),
 		         bad_domains);
-		g_clear_pointer (&bad_domains, g_free);
+		nm_clear_g_free (&bad_domains);
 	}
 
 	if (global_opt.become_daemon && !global_opt.debug) {
@@ -523,6 +524,7 @@ main (int argc, char *argv[])
 		                                          global_opt.dhcp4_hostname,
 		                                          global_opt.dhcp4_fqdn,
 		                                          NM_DHCP_HOSTNAME_FLAGS_FQDN_DEFAULT_IP4,
+		                                          global_opt.mud_url,
 		                                          client_id,
 		                                          NM_DHCP_TIMEOUT_DEFAULT,
 		                                          NULL,
@@ -596,7 +598,7 @@ main (int argc, char *argv[])
 	_LOGI (LOGD_CORE, "exiting");
 
 	nm_clear_g_source (&sd_id);
-	g_clear_pointer (&gl.main_loop, g_main_loop_unref);
+	nm_clear_pointer (&gl.main_loop, g_main_loop_unref);
 	return 0;
 }
 
